import {Component, EventEmitter, Input, OnChanges, OnDestroy, OnInit, Output, SimpleChanges, ViewChild} from '@angular/core';
import {FormBuilder, FormControl, FormGroup, Validators} from '@angular/forms';
import {AlertModal} from "../../../utils/modal/alert";
import {UserRegistryService} from "../../../services/user-registry.service";
import {EnvProperties} from "../../../utils/properties/env-properties";
import {properties} from "../../../../../environments/environment";
import {Role, Session, User} from "../../../login/utils/helper.class";
import {UserManagementService} from "../../../services/user-management.service";
import {Router} from "@angular/router";
import {StringUtils} from "../../../utils/string-utils.class";
import {NotificationService} from "../../../notifications/notification.service";
import {Subscription} from "rxjs";

declare var UIkit;

@Component({
  selector: 'role-users',
  templateUrl: 'role-users.component.html'
})
export class RoleUsersComponent implements OnInit, OnDestroy, OnChanges {
  
  @Input()
  public id: string;
  @Input()
  public type: string;
  @Input()
  public name: string;
  @Input()
  public link: string;
  @Input()
  public role: "member" | "manager" = "manager";
  @Input()
  public message: string = null;
  @Input()
  public emailComposer: Function;
  @Input()
  public notificationFn: Function;
  @Input()
  public inviteDisableMessage: string;
	@Output()
  public stickyEmitter: EventEmitter<boolean> = new EventEmitter<boolean>();
  public user: User = null;
  public active: any[] = [];
  public showActive: any[] = [];
  public pending: any[];
  public showPending: any[] = [];
  public showCurrent: boolean = true;
  public subs: any[] = [];
  public loadActive: boolean = true;
  public loadPending: boolean = true;
  public selectedUser: string = null;
  public invited: FormControl;
  public properties: EnvProperties = properties;
  public exists: boolean = true;
  public roleFb: FormGroup;
  /** Paging */
  activePage: number = 1;
  pendingPage: number = 1;
  pageSize: number = 10;
  /** Search */
  filterForm: FormGroup;
  @ViewChild('inviteModal') inviteModal: AlertModal;
  @ViewChild('deleteModal') deleteModal: AlertModal;
  @ViewChild('deletePendingModal') deletePendingModal: AlertModal;
  @ViewChild('createRoleModal') createRoleModal: AlertModal;
  
  constructor(private userRegistryService: UserRegistryService,
              private userManagementService: UserManagementService,
              private notificationService: NotificationService,
              private router: Router,
              private fb: FormBuilder) {
  }
  
  ngOnInit() {
    this.initForm();
    this.updateLists();
    this.userManagementService.getUserInfo().subscribe(user => {
      this.user = user;
    });
  }
  
  ngOnChanges(changes: SimpleChanges) {
    if (changes.role) {
      this.unsubscribe();
      this.initForm();
      this.updateLists();
    }
  }
  
  ngOnDestroy() {
    this.unsubscribe();
  }
  
  unsubscribe() {
    this.subs.forEach(sub => {
      if (sub instanceof Subscription) {
        sub.unsubscribe();
      }
    });
  }
  
  initForm() {
    this.filterForm = this.fb.group({
      active: this.fb.control(''),
      pending: this.fb.control('')
    });
    this.subs.push(this.filterForm.get('active').valueChanges.subscribe(value => {
      this.filterActiveBySearch(value);
    }));
    this.subs.push(this.filterForm.get('pending').valueChanges.subscribe(value => {
      this.filterPendingBySearch(value);
    }));
  }
  
  updateLists() {
    this.loadActive = true;
    this.loadPending = true;
    this.subs.push(this.userRegistryService.getActive(this.type, this.id, this.role).subscribe(users => {
      this.active = users;
			// users.forEach(user => this.active.push(user));
      this.filterActiveBySearch(this.filterForm.value.active);
      this.loadActive = false;
      this.exists = true;
    }, error => {
      this.active = [];
      this.showActive = [];
      if (error.status === 404) {
        this.exists = false;
      }
      this.loadActive = false;
    }));
    this.subs.push(this.userRegistryService.getPending(this.type, this.id, this.role).subscribe(users => {
      this.pending = users;
      this.filterPendingBySearch(this.filterForm.value.pending);
      this.loadPending = false;
    }, error => {
      this.pending = [];
      this.loadPending = false;
    }));
  }
  
  get currentActivePage(): any[] {
    if (this.showActive) {
      return this.showActive.slice((this.activePage - 1) * this.pageSize, this.activePage * this.pageSize);
    } else {
      return [];
    }
  }
  
  get currentPendingPage(): any[] {
    if (this.showPending) {
      return this.showPending.slice((this.pendingPage - 1) * this.pageSize, this.pendingPage * this.pageSize);
    } else {
      return [];
    }
  }
  
  openDeleteModal(item: any) {
    if (this.showCurrent) {
      this.selectedUser = item.email;
      this.deleteModal.alertTitle = 'Delete ' + this.role;
      this.deleteModal.open();
    } else {
      this.selectedUser = item;
      this.deletePendingModal.alertTitle = 'Cancel invitation';
      this.deletePendingModal.open();
    }
  }
  
  openInviteModal() {
    this.inviteModal.alertTitle = 'Invite ' + this.role;
    this.inviteModal.okButtonLeft = false;
    this.inviteModal.okButtonText = 'Send';
    this.invited = this.fb.control('', [Validators.required, Validators.email]);
    this.inviteModal.open();
  }
  
  openCreateRoleModal() {
    this.createRoleModal.alertTitle = 'Create group';
    this.createRoleModal.okButtonLeft = false;
    this.createRoleModal.okButtonText = 'Create';
    this.roleFb = this.fb.group({
      name: this.fb.control(Role.mapType(this.type) + '.' + this.id, Validators.required),
      description: this.fb.control(Role.mapType(this.type) + ' ' + this.id, Validators.required)
    });
    setTimeout(() => {
      this.roleFb.get('name').disable();
      this.roleFb.get('description').disable();
    }, 0);
    this.createRoleModal.open();
  }
  
  deleteActive() {
    this.loadActive = true;
    this.subs.push(this.userRegistryService.remove(this.type, this.id, this.selectedUser, this.role).subscribe(() => {
      this.active = this.active.filter(user => user.email != this.selectedUser);
      if (this.currentActivePage.length === 0) {
        this.activePage = 1;
      }
      this.filterActiveBySearch(this.filterForm.value.active);
      this.userManagementService.updateUserInfo();
      UIkit.notification(this.selectedUser + ' <b>is no longer</b> ' + this.role + ' of ' + this.name + ' Dashboard', {
        status: 'success',
        timeout: 6000,
        pos: 'bottom-right'
      });
      this.loadActive = false;
    }, error => {
      UIkit.notification('An error has occurred. Please try again later', {
        status: 'danger',
        timeout: 6000,
        pos: 'bottom-right'
      });
      this.loadActive = false;
    }));
  }
  
  deletePending() {
    this.loadPending = true;
    this.subs.push(this.userRegistryService.cancelInvitation(this.type, this.id, this.selectedUser, this.role).subscribe(() => {
      this.pending = this.pending.filter(user => user != this.selectedUser);
      this.filterPendingBySearch(this.filterForm.value.pending);
      if (this.currentPendingPage.length === 0) {
        this.pendingPage = 1;
      }
      UIkit.notification(StringUtils.capitalize(this.role) + ' invitation to ' + this.selectedUser + ' has been <b>canceled</b>', {
        status: 'success',
        timeout: 6000,
        pos: 'bottom-right'
      });
      this.loadPending = false;
    }, error => {
      UIkit.notification('An error has occurred. Please try again later', {
        status: 'danger',
        timeout: 6000,
        pos: 'bottom-right'
      });
      this.loadPending = false;
    }));
  }
  
  invite() {
    this.showCurrent = false;
    this.loadPending = true;
    this.selectedUser = this.invited.value;
    let details = {
      link: this.link,
      email: this.emailComposer(this.name, this.invited.value, this.role)
    }
    this.subs.push(this.userRegistryService.invite(this.type, this.id, details, this.role).subscribe(invitation => {
      if (!this.pending.includes(this.invited.value)) {
        this.pending.push(this.invited.value);
        this.pendingPage = Math.ceil(this.pending.length / this.pageSize);
        this.filterPendingBySearch(this.filterForm.value.pending);
      }
      if (this.notificationFn) {
        this.subs.push(this.notificationService.sendNotification(this.notificationFn(this.name, this.invited.value, this.role, invitation)).subscribe(notification => {
          UIkit.notification('A notification has been <b>sent</b> successfully', {
            status: 'success',
            timeout: 6000,
            pos: 'bottom-right'
          });
        }, error => {
          UIkit.notification('An error has occurred. Please try again later', {
            status: 'danger',
            timeout: 6000,
            pos: 'bottom-right'
          });
        }));
      }
      UIkit.notification(StringUtils.capitalize(this.role) + ' invitation to ' + this.selectedUser + ' has been <b>sent</b>', {
        status: 'success',
        timeout: 6000,
        pos: 'bottom-right'
      });
      this.loadPending = false;
    }, error => {
      UIkit.notification('An error has occurred. Please try again later', {
        status: 'danger',
        timeout: 6000,
        pos: 'bottom-right'
      });
      this.loadPending = false;
    }));
  }
  
  createGroup() {
    this.loadActive = true;
    this.loadPending = true;
    this.roleFb.get('name').enable();
    this.userRegistryService.createRole(this.type, this.id).subscribe(() => {
      UIkit.notification('Group has been <b> successfully created</b>', {
        status: 'success',
        timeout: 6000,
        pos: 'bottom-right'
      });
      this.updateLists();
    }, error => {
      UIkit.notification('An error has occurred. Please try again later', {
        status: 'danger',
        timeout: 6000,
        pos: 'bottom-right'
      });
      this.loadActive = false;
      this.loadPending = false;
    });
  }
  
  public get isCurator(): boolean {
    return this.isPortalAdmin || !!Session.isCurator(this.type, this.user);
  }
  
  public get isPortalAdmin(): boolean {
    return !!Session.isPortalAdministrator(this.user);
  }
  
  updateActivePage(event: any) {
    this.activePage = event.value;
  }
  
  updatePendingPage(event: any) {
    this.pendingPage = event.value;
  }
  
  private filterActiveBySearch(value: any) {
    this.showActive = this.active.filter(active => !value || active.email.includes(value));
    this.activePage = 1;
  }
  
  private filterPendingBySearch(value: any) {
    this.showPending = this.pending.filter(pending => !value || pending.includes(value));
    this.pendingPage = 1;
  }
}
